// This program is free software: you can use, modify and/or redistribute it
// under the terms of the simplified BSD License. You should have received a
// copy of this license along this program. If not, see
// <http://www.opensource.org/licenses/bsd-license.html>.
//
// Copyright (C) 2017, Javier Sánchez Pérez <jsanchez@ulpgc.es>
// All rights reserved.


#include "histogram.h"

/**
  *
  *  Function to compute the histogram for each channel
  * 
**/
void channel_histogram(
  unsigned char *I, //input image
  float *H,         //output histogram
  int    nx,        //number of columns
  int    ny,        //number of rows
  int    nz         //number of channels
)
{
  int size=nx*ny;

  //initialize the histogram
  for(int i=0;i<BIN_SIZE*nz;i++)
    H[i]=0;
  
  //compute normalized histogram
  for(int i=0;i<size;i++)
    for(int c=0;c<nz;c++){
      int p=(int)(BIN_RATE*I[i*nz+c]+0.5);
      H[c*BIN_SIZE+p]+=1.0;
    }
}

/**
  *
  *  Function to compute the normalized accumulated histogram for each channel
  * 
**/
void compute_channel_normalized_histogram(
  unsigned char *I, //input image
  float *H,         //output histogram
  int    nx,        //number of columns
  int    ny,        //number of rows
  int    nz         //number of channels
)
{
  float N=nx*ny;

  //initialize histogram
  for(int i=0;i<BIN_SIZE*nz;i++)
    H[i]=0;
  
  //compute histogram
  for(int i=0;i<N;i++)
    for(int c=0;c<nz;c++)
    {
      int p=(int)(BIN_RATE*I[i*nz+c]+0.5);
      H[c*BIN_SIZE+p]+=1.0;
    }
    
  //normalize histogram
  for(int i=0;i<BIN_SIZE*nz;i++)
    H[i]/=N;

  //accumulate histogram
  for(int c=0;c<nz;c++)
    for(int i=1;i<BIN_SIZE;i++)
      H[c*BIN_SIZE+i]+=H[c*BIN_SIZE+i-1];  
}


/**
  *
  *  Function to compute the normalized accumulated histogram for 
  *  grayscale images
  * 
**/
void compute_normalized_histogram(
  float *I,  //input image
  float *H,  //output histogram
  int    nx, //number of columns
  int    ny  //number of rows
)
{
  float N=nx*ny;

  //initialize histogram
  for(int i=0;i<BIN_SIZE;i++)
    H[i]=0;
  
  //compute histogram
  for(int i=0;i<N;i++)
  {
    int p=(int)(BIN_RATE*I[i]+0.5);
    
    if(p<0) 
      p=0;
    else if(p>=BIN_SIZE) 
      p=BIN_SIZE-1;
    
    H[p]+=1.0;
  }

  //normalize histogram
  for(int i=0;i<BIN_SIZE;i++)
    H[i]/=N;

  //accumulate histogram
  for(int i=1;i<BIN_SIZE;i++)
    H[i]+=H[i-1];  
}
