// This program is free software: you can use, modify and/or redistribute it
// under the terms of the simplified BSD License. You should have received a
// copy of this license along this program. If not, see
// <http://www.opensource.org/licenses/bsd-license.html>.
//
// Copyright (C) 2017, Javier Sánchez Pérez <jsanchez@ulpgc.es>
// All rights reserved.


#include <stdio.h>
#include <random>

/**
  *
  *  Function to read a video in raw format
  * 
**/
size_t read_video(
  char *name,       //file name
  unsigned char *I, //video to write
  int size          //size of the video
) 
{
  FILE* file=fopen(name, "rb");
  if(file != NULL)
  {
    size_t result=fread(I, sizeof(unsigned char), size, file);
    fclose(file);
    return result;
  }
  else return 0;
}


/**
  *
  *  Function to read a video in raw format
  * 
**/
size_t write_video(
  char *name,       //file name
  unsigned char *I, //video to write
  int size          //size of the video
) 
{
  FILE* file=fopen(name, "wb");
  if(file != NULL)
  {
    size_t result=fwrite(I, sizeof(unsigned char), size, file);
    fclose(file);
    return result;
  }
  else return 0;
}


/**
  *
  *  Function to add noise to an image
  * 
**/
void add_noise(
  float *u,    //input image
  float *v,    //output noisy image
  float sigma, //Gaussian standard deviation
  int   size   //number of pixels
) 
{  
  std::default_random_engine generator;
  std::normal_distribution<float> dist(0, sigma);

  #pragma omp parallel for
  for (int i=0; i< size; i++) 
    v[i] =  u[i] + dist(generator);
}


/**
  *
  *  Function to convert an rgb image to grayscale levels
  * 
**/
void rgb2gray(
  unsigned char *rgb, //input color image
  float *gray, //output grayscale image
  int size     //number of pixels
)
{
  #pragma omp parallel for
  for(int i=0;i<size;i++)
    gray[i]=(0.2989*rgb[i*3]+0.5871*rgb[i*3+1]+0.1140*rgb[i*3+2]);
}
